@@ -4,6 +4,7 @@
 from datetime import datetime, timezone, timedelta
 from pathlib import Path
 import warnings
+import numpy as np
 import pandas as pd
 from covsirphy.util.argument import find_args
 from covsirphy.util.error import deprecate, DBLockedError, NotDBLockedError, UnExpectedValueError
@@ -29,7 +30,7 @@ class DataLoader(Term):
 
     Args:
         directory (str or pathlib.Path): directory to save downloaded datasets
-        update_interval (int or None): update interval of downloaded or None (avoid downloading)
+        update_interval (int or None): update interval of downloading dataset or None (avoid downloading)
         basename_dict (dict[str, str]): basename of downloaded CSV files,
             "covid19dh": COVID-19 Data Hub (default: covid19dh.csv),
             "owid": Our World In Data (default: ourworldindata.csv),
@@ -42,6 +43,8 @@ class DataLoader(Term):
         If @update_interval (not None) hours have passed since the last update of downloaded datasets,
         the dawnloaded datasets will be updated automatically.
         When we do not use datasets of remote servers, set @update_interval as None.
+        Note that the outputs of .japan() and .pyramid() are not locked by .lock().
+        So, they are not influenced by @update_interval.
 
     Note:
         If @verbose is 0, no discription will be shown.
@@ -53,8 +56,8 @@ class DataLoader(Term):
         # Directory
         try:
             self.dir_path = Path(directory)
-        except TypeError:
-            raise TypeError(f"@directory should be a path-like object, but {directory} was applied.")
+        except TypeError as e:
+            raise TypeError(f"@directory should be a path-like object, but {directory} was applied.") from e
         self.update_interval = self._ensure_natural_int(
             update_interval, name="update_interval", include_zero=True, none_ok=True)
         # Create the directory if not exist
@@ -283,7 +286,7 @@ class DataLoader(Term):
 
     def lock(self, date, country, province, iso3=None,
              confirmed=None, fatal=None, recovered=None, population=None, tests=None,
-             product=None, vaccinations=None, vaccinated_once=None, vaccinated_full=None,
+             product=None, vaccinations=None, vaccinations_boosters=None, vaccinated_once=None, vaccinated_full=None,
              oxcgrt_variables=None, mobility_variables=None):
         """
         Lock the local database, specifying columns which has date and area information.
@@ -300,6 +303,7 @@ class DataLoader(Term):
             tests (str or None): the number of tests
             product (str or None): vaccine product names
             vaccinations (str or None): cumulative number of vaccinations
+            vaccinations_boosters (str or None): cumulative number of booster vaccinations
             vaccinated_once (str or None): cumulative number of people who received at least one vaccine dose
             vaccinated_full (str or None): cumulative number of people who received all doses prescrived by the protocol
             oxcgrt_variables (list[str] or None): list of variables for OxCGRTData
@@ -321,14 +325,14 @@ class DataLoader(Term):
         # All variables
         variables = [
             self.ISO3, self.C, self.F, self.R, self.N, self.TESTS,
-            self.PRODUCT, self.VAC, self.V_ONCE, self.V_FULL,
+            self.PRODUCT, self.VAC, self.VAC_BOOSTERS, self.V_ONCE, self.V_FULL,
             *self._oxcgrt_cols, *self._mobility_cols,
         ]
         id_dict = {date: self.DATE, country: self.COUNTRY, province: self.PROVINCE}
         rename_dict = {
             **id_dict, iso3: self.ISO3,
             confirmed: self.C, fatal: self.F, recovered: self.R, population: self.N,
-            tests: self.TESTS, product: self.PRODUCT, vaccinations: self.VAC,
+            tests: self.TESTS, product: self.PRODUCT, vaccinations: self.VAC, vaccinations_boosters: self.VAC_BOOSTERS,
             vaccinated_once: self.V_ONCE, vaccinated_full: self.V_FULL,
         }
         # Local database
@@ -431,7 +435,7 @@ class DataLoader(Term):
         self._set_date_location(remote_df)
         remote_df = remote_df.set_index(self._id_cols)
         # Update the current database
-        df = df.replace(0, None).combine_first(remote_df).reset_index().set_index(self._id_cols)
+        df = df.replace(0, np.nan).combine_first(remote_df).reset_index().set_index(self._id_cols)
         # Update citations
         cite_dict = {k: [*v, handler.CITATION] if k in remote_df else v for (k, v) in cite_dict.items()}
         return (df, cite_dict, handler)
@@ -559,7 +563,7 @@ class DataLoader(Term):
         self._read_dep(**kwargs)
         filename = self._filename_dict["japan"]
         if self._japan_data is None:
-            force = self._download_necessity(filename=filename)
+            force = True if self.update_interval is None else self._download_necessity(filename=filename)
             self._japan_data = JapanData(filename=filename, force=force, verbose=self._verbose)
         return self._japan_data
 
@@ -606,9 +610,9 @@ class DataLoader(Term):
             covsirphy.VaccineData: dataset regarding vaccinations
         """
         self._read_dep(**kwargs)
-        v_cols = [self.PRODUCT, self.VAC, self.V_ONCE, self.V_FULL]
+        v_cols = [self.PRODUCT, self.VAC, self.VAC_BOOSTERS, self.V_ONCE, self.V_FULL]
         df, citations = self._auto_lock(variables=v_cols)
-        return VaccineData(data=df.dropna(subset=v_cols), citation="\n".join(citations))
+        return VaccineData(data=df.dropna(subset=[self.VAC]), citation="\n".join(citations))
 
     def mobility(self):
         """
